package com.hai.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hai.common.Base64Util;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.AESEncodeUtil;
import com.hai.common.security.UserCenter;
import com.hai.common.utils.RedisUtil;
import com.hai.config.CommonSysConst;
import com.hai.dao.HighDiscountPackageAgentCodeMapper;
import com.hai.dao.HighDiscountPackageAgentRecordMapper;
import com.hai.dao.HighDiscountPackageAgentRelMapper;
import com.hai.entity.*;
import com.hai.model.HighUserModel;
import com.hai.model.UserInfoModel;
import com.hai.service.HighAgentService;
import com.hai.service.HighDiscountPackageActualService;
import com.hai.service.HighDiscountPackageAgentService;
import com.hai.service.HighDiscountPackageService;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service("highDiscountPackageAgentService")
public class HighDiscountPackageAgentServiceImpl implements HighDiscountPackageAgentService {

    @Resource
    private HighDiscountPackageAgentRecordMapper discountPackageAgentRecordMapper;

    @Resource
    private HighDiscountPackageAgentRelMapper discountPackageAgentRelMapper;

    @Resource
    private HighDiscountPackageAgentCodeMapper discountPackageAgentCodeMapper;

    @Resource
    private HighDiscountPackageActualService discountPackageActualService;

    @Resource
    private HighDiscountPackageAgentService discountPackageAgentService;
    @Resource
    private HighDiscountPackageService discountPackageService;

    @Resource
    private HighAgentService highAgentService;

    @Resource
    private RedisUtil redisUtil;

    @Autowired
    private UserCenter userCenter;

    @Override
    public void insertDiscountPackageAgentRel(HighDiscountPackageAgentRel discountPackageAgentRel) {
        discountPackageAgentRelMapper.insert(discountPackageAgentRel);
    }

    @Override
    public void updateDiscountPackageAgentRel(HighDiscountPackageAgentRel discountPackageAgentRel) {
        discountPackageAgentRelMapper.updateByPrimaryKey(discountPackageAgentRel);
    }

    @Override
    public HighDiscountPackageAgentRel getDetailRelById(Long id) {
        return discountPackageAgentRelMapper.selectByPrimaryKey(id);
    }

    @Override
    public HighDiscountPackageAgentRel getDetailRelByMap(Map<String, Object> map) {

        HighDiscountPackageAgentRelExample example = new HighDiscountPackageAgentRelExample();
        HighDiscountPackageAgentRelExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(map, "agentId") != null) {
            criteria.andAgentIdEqualTo(MapUtils.getLong(map, "agentId"));
        }
        if (MapUtils.getLong(map, "discountPackageId") != null) {
            criteria.andDiscountPackageIdEqualTo(MapUtils.getLong(map, "discountPackageId"));
        }
        if (MapUtils.getInteger(map, "type") != null) {
            criteria.andTypeEqualTo(MapUtils.getInteger(map, "type"));
        }

        List<HighDiscountPackageAgentRel> list = discountPackageAgentRelMapper.selectByExample(example);

        if (list.size() > 0) {
            return list.get(0);
        }

        return null;
    }

    @Override
    public List<HighDiscountPackageAgentRel> getListRel(Map<String, Object> param) {

        HighDiscountPackageAgentRelExample example = new HighDiscountPackageAgentRelExample();
        HighDiscountPackageAgentRelExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(param, "agentId") != null) {
            criteria.andAgentIdEqualTo(MapUtils.getLong(param, "agentId"));
        }
        if (MapUtils.getInteger(param, "type") != null) {
            criteria.andTypeEqualTo(MapUtils.getInteger(param, "type"));
        }

        return discountPackageAgentRelMapper.selectByExample(example);
    }

    @Override
    public void insertDiscountPackageAgentRecord(HighDiscountPackageAgentRecord discountPackageAgentRecord) {
        discountPackageAgentRecordMapper.insert(discountPackageAgentRecord);
    }

    @Override
    public void updateDiscountPackageAgentRecord(HighDiscountPackageAgentRecord discountPackageAgentRecord) {
        discountPackageAgentRecordMapper.updateByPrimaryKey(discountPackageAgentRecord);
    }

    @Override
    public HighDiscountPackageAgentRecord getDetailRecordById(Long id) {
        return discountPackageAgentRecordMapper.selectByPrimaryKey(id);
    }

    @Override
    public HighDiscountPackageAgentRecord getDetailRecordByMap(Map<String, Object> map) {
        return null;
    }

    @Override
    public List<HighDiscountPackageAgentRecord> getListRecord(Map<String, Object> param) {

        HighDiscountPackageAgentRecordExample example = new HighDiscountPackageAgentRecordExample();
        HighDiscountPackageAgentRecordExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(param, "agentId") != null) {
            criteria.andAgentIdEqualTo(MapUtils.getLong(param, "agentId"));
        }
        if (MapUtils.getLong(param, "discountPackageId") != null) {
            criteria.andDiscountPackageIdEqualTo(MapUtils.getLong(param, "discountPackageId"));
        }
        if (MapUtils.getInteger(param, "type") != null) {
            criteria.andTypeEqualTo(MapUtils.getInteger(param, "type"));
        }

        return discountPackageAgentRecordMapper.selectByExample(example);
    }

    @Override
    @Transactional(propagation= Propagation.REQUIRED)
    public void generateDiscountPackageAgentCode(JSONObject object)  throws Exception{
        List<HighDiscountPackageActual> discountPackageActualList = discountPackageActualService.getListByPackageId(object.getInteger("discountPackageId") , 1);

        if (object.getInteger("num") > discountPackageActualList.size()) {
            throw ErrorHelp.genException(SysCode.System , ErrorCode.COMMON_ERROR , "优惠券包库存不足");
        }
        Map<String,Object> mapCode = new HashMap<>();
        HighDiscountPackageAgentCode discountPackageAgentCode ;
        mapCode.put("type" , "DISCOUNT_PACKAGE");
        int percent;

        for (int i = 1; i <= object.getInteger("num"); i++) {

            // 生成百分比
            percent = i*100/object.getInteger("num");
            redisUtil.set(object.getString("serialNumber") , percent , 3600);

            HighDiscountPackageActual discountPackageActual =  discountPackageActualList.get(i);
            discountPackageAgentCode = new HighDiscountPackageAgentCode();
            discountPackageAgentCode.setAgentId(object.getLong("agentId"));
            discountPackageAgentCode.setAgentName(object.getString("agentName"));
            discountPackageAgentCode.setDiscountPackageId(object.getLong("discountPackageId"));
            discountPackageAgentCode.setDiscountPackageName(object.getString("discountPackageName"));
            discountPackageAgentCode.setDiscountPackageActualId(discountPackageActual.getId().longValue());
            discountPackageAgentCode.setCreateTime(new Date());
            discountPackageAgentCode.setStatus(1);
            insertDiscountPackageAgentCode(discountPackageAgentCode);
            // 二维码参数
            mapCode.put("id", discountPackageActual.getId());
            String code = Base64Util.encode(AESEncodeUtil.aesEncrypt(JSON.toJSONString(mapCode)));
            // 生成二维码
            String qrCodeUrl = CommonSysConst.getSysConfig().getHsgDomainName() + "/hsgH5?accountId=0000022&key=" + object.getLong("discountPackageId") + "&code=" + code;
            discountPackageAgentCode.setQrCodeUrl(qrCodeUrl);
            discountPackageAgentCode.setCode(code);
            discountPackageAgentCode.setDiscountAgentBatchId(object.getLong("serialNumber"));
            updateDiscountPackageAgentCode(discountPackageAgentCode);
            discountPackageActual.setStatus(2);
            discountPackageActual.setExt3(object.getString("serialNumber"));
            discountPackageActualService.updateHighDiscountPackageActual(discountPackageActual);
        }

        Map<String , Object> map = new HashMap<>();
        map.put("agentId" , object.getLong("agentId"));
        map.put("discountPackageId" , object.getLong("discountPackageId"));
        map.put("type" , 2);
        HighDiscountPackageAgentRel discountPackageAgentRel = discountPackageAgentService.getDetailRelByMap(map);
        HighDiscountPackageAgentRecord discountPackageAgentRecord = new HighDiscountPackageAgentRecord();


        if (discountPackageAgentRel == null) {
            discountPackageAgentRel = new HighDiscountPackageAgentRel();
            discountPackageAgentRel.setStatus(1);
            discountPackageAgentRel.setType(2);
            discountPackageAgentRel.setAgentId(object.getLong("agentId"));
            discountPackageAgentRel.setAgentName(object.getString("agentName"));
            discountPackageAgentRel.setDiscountPackageId(object.getLong("discountPackageId"));
            discountPackageAgentRel.setDiscountPackageTitle(object.getString("discountPackageName"));
            discountPackageAgentRel.setSalesCount(0);
            discountPackageAgentRel.setStockCount(0);
            discountPackageAgentRel.setCreateTime(new Date());
            discountPackageAgentRel.setUpdateTime(new Date());
        }
        discountPackageAgentRel.setStockCount(object.getInteger("num") + discountPackageAgentRel.getStockCount());

        if (discountPackageAgentRel.getId() == null) {
            discountPackageAgentService.insertDiscountPackageAgentRel(discountPackageAgentRel);
        } else {
            discountPackageAgentRel.setUpdateTime(new Date());
            discountPackageAgentRel.setType(2);
            discountPackageAgentService.updateDiscountPackageAgentRel(discountPackageAgentRel);
        }

        discountPackageAgentRecord.setAgentId(object.getLong("agentId"));
        discountPackageAgentRecord.setAgentName(object.getString("agentName"));
        discountPackageAgentRecord.setDiscountPackageId(object.getLong("discountPackageId"));
        discountPackageAgentRecord.setDiscountPackageTitle(object.getString("discountPackageName"));
        discountPackageAgentRecord.setCreateTime(new Date());
        discountPackageAgentRecord.setUpdateTime(new Date());
        discountPackageAgentRecord.setStatus(1);
        discountPackageAgentRecord.setType(2);
        discountPackageAgentRecord.setCount(object.getInteger("num"));
        discountPackageAgentRecord.setAfterCount(discountPackageAgentRel.getStockCount());
        discountPackageAgentRecord.setFrontCount(discountPackageAgentRel.getStockCount() - object.getInteger("num"));

        discountPackageAgentService.insertDiscountPackageAgentRecord(discountPackageAgentRecord);

    }

    @Override
    public void insertDiscountPackageAgentCode(HighDiscountPackageAgentCode discountPackageAgentCode) {
        discountPackageAgentCodeMapper.insert(discountPackageAgentCode);
    }

    @Override
    public void updateDiscountPackageAgentCode(HighDiscountPackageAgentCode discountPackageAgentCode) {
        discountPackageAgentCodeMapper.updateByPrimaryKey(discountPackageAgentCode);
    }

    @Override
    public HighDiscountPackageAgentCode findDiscountPackageAgentCode(String code) {
        HighDiscountPackageAgentCodeExample example = new HighDiscountPackageAgentCodeExample();
        HighDiscountPackageAgentCodeExample.Criteria criteria = example.createCriteria();

        criteria.andCodeEqualTo(code);

        List<HighDiscountPackageAgentCode> list = discountPackageAgentCodeMapper.selectByExample(example);

        if (list.size() > 0) {
            return list.get(0);
        }

        return null;
    }

    @Override
    public List<HighDiscountPackageAgentCode> getDiscountPackageAgentCodeList(Map<String, Object> map) {
        HighDiscountPackageAgentCodeExample example = new HighDiscountPackageAgentCodeExample();
        HighDiscountPackageAgentCodeExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(map, "agentId") != null) {
            criteria.andAgentIdEqualTo(MapUtils.getLong(map, "agentId"));
        }
        if (MapUtils.getLong(map, "discountPackageId") != null) {
            criteria.andDiscountPackageIdEqualTo(MapUtils.getLong(map, "discountPackageId"));
        }

        return discountPackageAgentCodeMapper.selectByExample(example);
    }

    @Override
    @Transactional(propagation= Propagation.REQUIRED)
    public void recoveryDiscountPackage(JSONObject object) {

        // 用户信息
        UserInfoModel userInfoModel = userCenter.getSessionModel(UserInfoModel.class);

        // 查询代理商
        HighAgent agent = highAgentService.findByAgentMsgId(object.getLong("agentId"));
        // 查询优惠券包
        HighDiscountPackage discountPackage = discountPackageService.findDiscountPackageById(object.getInteger("discountPackageId"));
        // 判断代理商状态是否正常
        if (agent.getStatus() != 1) {
            throw ErrorHelp.genException(SysCode.System , ErrorCode.COMMON_ERROR , "当前代理商不支持回收库存！");
        }
        // 查询代理商与优惠券包的关联关系
        Map<String , Object> map = new HashMap<>();
        map.put("agentId" , object.getLong("agentId"));
        map.put("discountPackageId" , object.getLong("discountPackageId"));
        map.put("type" , 1);

        // 判断是否库存足够回收
        HighDiscountPackageAgentRel discountPackageAgentRel = discountPackageAgentService.getDetailRelByMap(map);
        if (discountPackageAgentRel != null && discountPackageAgentRel.getStockCount() < object.getInteger("num") ) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "库存数量不足");
        }



        // 录入回收记录
        HighDiscountPackageAgentRecord discountPackageAgentRecord = new HighDiscountPackageAgentRecord();
        discountPackageAgentRecord.setAgentId(object.getLong("agentId"));
        discountPackageAgentRecord.setAgentName(agent.getAgentName());
        discountPackageAgentRecord.setDiscountPackageId(object.getLong("discountPackageId"));
        discountPackageAgentRecord.setDiscountPackageTitle(discountPackage.getTitle());
        discountPackageAgentRecord.setCreateTime(new Date());
        discountPackageAgentRecord.setUpdateTime(new Date());
        discountPackageAgentRecord.setStatus(1);
        discountPackageAgentRecord.setType(3);
        discountPackageAgentRecord.setOpId(userInfoModel.getSecUser().getId());
        discountPackageAgentRecord.setOpName(userInfoModel.getSecUser().getLoginName());
        discountPackageAgentRecord.setCount(object.getInteger("num"));
        discountPackageAgentRecord.setAfterCount(discountPackageAgentRel.getStockCount());
        discountPackageAgentRecord.setFrontCount(discountPackageAgentRel.getStockCount() - object.getInteger("num"));
        discountPackageAgentRecord.setExt1("代理商id：" + object.getLong("agentId") + "-优惠券包名称" + discountPackage.getTitle() + "-回收数量" + object.getInteger("num"));
        discountPackageAgentService.insertDiscountPackageAgentRecord(discountPackageAgentRecord);

        // 回收库存
        discountPackageAgentRel.setStockCount(discountPackageAgentRel.getStockCount()-object.getInteger("num"));
        discountPackageAgentService.updateDiscountPackageAgentRel(discountPackageAgentRel);

        if (agent.getParentId() == null) {
            throw ErrorHelp.genException(SysCode.System , ErrorCode.COMMON_ERROR , "当前代理商不支持回收库存!");
        }

        // 查询上级代理商
        Map<String , Object> mapParent = new HashMap<>();
        mapParent.put("agentId" , agent.getParentId());
        mapParent.put("discountPackageId" , object.getLong("discountPackageId"));
        mapParent.put("type" , 1);
        HighDiscountPackageAgentRel discountPackageAgentRelParent = discountPackageAgentService.getDetailRelByMap(mapParent);

        if (discountPackageAgentRelParent == null) {
            throw ErrorHelp.genException(SysCode.System , ErrorCode.COMMON_ERROR , "未查询到相关信息！");
        }

        // 录入回收记录
        HighDiscountPackageAgentRecord discountPackageAgentRecordParent = new HighDiscountPackageAgentRecord();
        discountPackageAgentRecordParent.setAgentId(object.getLong("agentId"));
        discountPackageAgentRecordParent.setAgentName(agent.getAgentName());
        discountPackageAgentRecordParent.setDiscountPackageId(object.getLong("discountPackageId"));
        discountPackageAgentRecordParent.setDiscountPackageTitle(discountPackage.getTitle());
        discountPackageAgentRecordParent.setCreateTime(new Date());
        discountPackageAgentRecordParent.setUpdateTime(new Date());
        discountPackageAgentRecordParent.setStatus(1);
        discountPackageAgentRecordParent.setType(3);
        discountPackageAgentRecordParent.setOpId(userInfoModel.getSecUser().getId());
        discountPackageAgentRecordParent.setOpName(userInfoModel.getSecUser().getLoginName());
        discountPackageAgentRecordParent.setCount(object.getInteger("num"));
        discountPackageAgentRecordParent.setAfterCount(discountPackageAgentRelParent.getStockCount());
        discountPackageAgentRecordParent.setFrontCount(discountPackageAgentRelParent.getStockCount() + object.getInteger("num"));
        discountPackageAgentRecordParent.setExt1("代理商id：" + object.getLong("agentId") + "-优惠券包名称" + discountPackage.getTitle() + "-回收数量" + object.getInteger("num"));
        discountPackageAgentService.insertDiscountPackageAgentRecord(discountPackageAgentRecordParent);

        // 上级回收库存
        discountPackageAgentRelParent.setStockCount(discountPackageAgentRelParent.getStockCount()+object.getInteger("num"));
        discountPackageAgentService.updateDiscountPackageAgentRel(discountPackageAgentRelParent);

    }
}
