package com.hai.service.impl;

import com.hai.dao.HighStoreUserMapper;
import com.hai.entity.HighMerchantStore;
import com.hai.entity.HighStoreUser;
import com.hai.entity.HighStoreUserExample;
import com.hai.entity.HighUser;
import com.hai.service.HighMerchantStoreService;
import com.hai.service.HighStoreUserService;
import com.hai.service.HighUserService;
import org.apache.commons.collections4.MapUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service("highStoreUserService")
public class HighStoreUserServiceImpl implements HighStoreUserService {

    @Resource
    private HighStoreUserMapper highStoreUserMapper;

    @Resource
    private HighMerchantStoreService highMerchantStoreService;

    @Resource
    private HighUserService highUserService;

    @Override
    public void insertStoreUser(HighStoreUser storeUser) {
        highStoreUserMapper.insert(storeUser);
    }

    @Override
    public void updateStoreUser(HighStoreUser storeUser) {
        highStoreUserMapper.updateByPrimaryKey(storeUser);
    }

    @Override
    public HighStoreUser findStoreUser(Map<String, Object> map) {
        HighStoreUserExample example = new HighStoreUserExample();
        HighStoreUserExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(map, "userId") != null) {
            criteria.andUserIdEqualTo(MapUtils.getLong(map, "userId"));
        }

        List<HighStoreUser> list = highStoreUserMapper.selectByExample(example);

        if (!list.isEmpty()) {
            return list.get(0);
        }


        return null;
    }

    @Override
    public List<HighStoreUser> getStoreUserList(Map<String, Object> map) {
        HighStoreUserExample example = new HighStoreUserExample();
        HighStoreUserExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(map, "userId") != null) {
            criteria.andUserIdEqualTo(MapUtils.getLong(map, "userId"));
        }
        if (MapUtils.getLong(map, "storeId") != null) {
            criteria.andUserIdEqualTo(MapUtils.getLong(map, "storeId"));
        }
        return highStoreUserMapper.selectByExample(example);
    }

    @Override
    public void storeUserRel(Long userId, Long storeId) {
        Map<String , Object> map = new HashMap<>();
        map.put("userId" , userId);

        HighStoreUser storeUser = findStoreUser(map);

        if (storeUser == null) {

            HighMerchantStore merchantStore = highMerchantStoreService.getMerchantStoreById(storeId);
            HighUser highUser = highUserService.findByUserId(userId);

            if (merchantStore != null && highUser != null) {
                storeUser = new HighStoreUser();
                storeUser.setStoreName(merchantStore.getStoreName());
                storeUser.setStoreId(storeId);
                storeUser.setUserPhone(highUser.getPhone());
                storeUser.setUserName(highUser.getName());
                storeUser.setCreateTime(new Date());
                storeUser.setUpdateTime(new Date());
                storeUser.setStatus(1);

                insertStoreUser(storeUser);
            }

        }

    }
}
