package com.v1.controller.business;

import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.config.PetroConfig;
import com.hai.entity.*;
import com.hai.model.ResponseData;
import com.hai.openApi.service.ApiCouponService;
import com.hai.openApi.service.ApiOrderService;
import com.hai.order.type.OrderProductType;
import com.hai.service.ApiIpAddressService;
import com.hai.service.ApiMerchantsService;
import com.hai.service.HighOrderService;
import com.hai.service.SecConfigService;
import com.v1.config.ToolConfig;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

@Controller
@RequestMapping(value = "/apiCoupon")
@Api(value = "卡券业务")
public class ApiCouponController {

    private static Logger log = LoggerFactory.getLogger(ApiCouponController.class);

    @Resource
    private ApiMerchantsService apiMerchantsService;

    @Resource
    private ApiCouponService apiCouponService;

    @Resource
    private SecConfigService secConfigService;

    @Resource
    private ApiIpAddressService apiIpAddressService;

    @Resource
    private ApiOrderService apiOrderService;

    @Resource
    private PetroConfig petroConfig;



    @Resource
    private HighOrderService highOrderService;

    @RequestMapping(value = "/getCouponList", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "获取卡券列表")
    public ResponseData getCouponList(@RequestBody JSONObject object , HttpServletRequest request ) {
        try {

            String ip = "";
            // 有的user可能使用代理，为处理用户使用代理的情况，使用x-forwarded-for
            if  (request.getHeader("x-forwarded-for") == null)  {
                ip = request.getRemoteAddr();
            }  else  {
                ip = request.getHeader("x-forwarded-for");
            }

            if (!apiIpAddressService.validationIpAddressLegal(object.getString("merchId") , ip)) {
                log.error("getRechargeProduct error!", "非法ip地址，请联系管理人员！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.IP_ERROR, "非法ip地址，请联系管理人员！");
            }

            if (
                    StringUtils.isBlank(object.getString("merchId")) ||
                            StringUtils.isBlank(object.getString("timetable")) ||
                            StringUtils.isBlank(object.getString("sign"))
            ) {
                log.error("getRechargeProduct error!", "请求参数校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR);
            }

            ApiMerchants apiMerchants = apiMerchantsService.findByMchId(object.getString("merchId"));

            if (apiMerchants == null) {
                log.error("getRechargeProduct error!", "帐户错误！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.ACCOUNT_NOT_EXIST);
            }

            if (!secConfigService.isConfig(OrderProductType.PRODUCT_TYPE18.getApiCode(), object.getString("merchId"))) {
                log.error("getAllCities error!", "当前帐户无权限查询！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MERCHANT_NOT_AUTHORIZED, "");
            }

            if (!ToolConfig.timetableCheck(15L , object.getLong("timetable"))) {
                log.error("getRechargeProduct error!", "请求时间超过15分钟！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.TIME_OUT);
            }

            Map<String , Object> map = new HashMap<>();
            map.put("apiKey" , apiMerchants.getApiKey());
            map.put("timetable" , object.getString("timetable"));
            map.put("merchId" , object.getString("merchId"));

            if (!secConfigService.isSignVerify(map , object.getString("sign"))) {
                log.error("getRechargeProduct error!", "签名校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.SIGN_VERIFY);
            }

            return ResponseMsgUtil.success(apiCouponService.getCouponList(new HashMap<>()));

        } catch (Exception e) {
            log.error("HighOpenApiController --> getRechargeProduct() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value = "/couponDetail", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "查询卡券详情")
    public ResponseData couponDetail(@RequestBody JSONObject object , HttpServletRequest request) {
        try {

            String ip;
            // 有的user可能使用代理，为处理用户使用代理的情况，使用x-forwarded-for
            if  (request.getHeader("x-forwarded-for") == null)  {
                ip = request.getRemoteAddr();
            }  else  {
                ip = request.getHeader("x-forwarded-for");
            }

            if (!apiIpAddressService.validationIpAddressLegal(object.getString("merchId") , ip)) {
                log.error("getRechargeProduct error!", "非法ip地址，请联系管理人员！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.IP_ERROR, "非法ip地址，请联系管理人员！");
            }

            if (
                    StringUtils.isBlank(object.getString("mchOrderNo")) ||
                            StringUtils.isBlank(object.getString("sign")) ||
                            StringUtils.isBlank(object.getString("timetable")) ||
                            StringUtils.isBlank(object.getString("merchId"))
            ) {
                log.error("createOrder error!", "请求参数校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR);
            }

            ApiMerchants apiMerchants = apiMerchantsService.findByMchId(object.getString("merchId"));

            if (apiMerchants == null) {
                log.error("getRechargeProduct error!", "帐户错误！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.ACCOUNT_NOT_EXIST);
            }

            if (!secConfigService.isConfig(OrderProductType.PRODUCT_TYPE18.getApiCode(), object.getString("merchId"))) {
                log.error("getAllCities error!", "当前帐户无权限查询！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MERCHANT_NOT_AUTHORIZED, "");
            }

            if (!ToolConfig.timetableCheck(15L , object.getLong("timetable"))) {
                log.error("getRechargeProduct error!", "请求时间超过15分钟！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.TIME_OUT);
            }

            Map<String , Object> map = new HashMap<>();
            map.put("mchOrderNo" , object.getString("mchOrderNo"));
            map.put("apiKey" , apiMerchants.getApiKey());
            map.put("timetable" , object.getString("timetable"));
            map.put("merchId" , object.getString("merchId"));

            if (!secConfigService.isSignVerify(map, object.getString("sign"))) {
                log.error("createOrder error!", "签名校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.SIGN_VERIFY);
            }


            ApiOrder apiOrder = apiOrderService.findByOrderNo(object.getString("mchOrderNo") , null);


            HighOrder highOrder = highOrderService.getOrderByOrderNo(apiOrder.getOrderNo());
            HighChildOrder childOrder = highOrder.getHighChildOrderList().get(0);

            // 派发停车券
            ApiCoupon apiCoupon = apiCouponService.findCoupon(childOrder.getGoodsSpecName());

            if (apiCoupon.getType().equals(1)) {
                PetroConfig.init(1);
            } else {
                PetroConfig.init(2);
            }

            JSONObject jsonObject = petroConfig.couponDetail(apiOrder.getExt3());

            if (!jsonObject.getString("code").equals("1")) {
                throw ErrorHelp.genException(SysCode.System , ErrorCode.COMMON_ERROR , "当前订单未查询到！");
            }

            String data = PetroConfig.decrypt(jsonObject.getString("biz_content"));

            JSONObject jsonData = JSONObject.parseObject(data);
            jsonData.put("out_trade_no" , object.getString("mchOrderNo"));

            return ResponseMsgUtil.success(jsonData);

        } catch (Exception e) {
            log.error("HighOpenApiController --> createOrder() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


}
